@@ -8,30 +8,43 @@ import (
 )
 
 func Test_initConfig(t *testing.T) {
-	t.Parallel()
 	tests := []struct {
 		name    string
+		xdg     string
 		cfgFile string
 		wantErr bool
 	}{
 		{
 			name:    "test-ov.yaml",
+			xdg:     "",
 			cfgFile: "ov.yaml",
 			wantErr: false,
 		},
 		{
 			name:    "test-ov-less.yaml",
+			xdg:     "",
 			cfgFile: "ov-less.yaml",
 			wantErr: false,
 		},
 		{
 			name:    "no-file.yaml",
+			xdg:     "",
 			cfgFile: "no-file.yaml",
 			wantErr: true,
 		},
+		{
+			name:    "not found",
+			xdg:     "dummy",
+			cfgFile: "",
+			wantErr: false,
+		},
 	}
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
+			if tt.xdg != "" {
+				t.Setenv("XDG_CONFIG_HOME", tt.xdg)
+			}
+
 			cfgFile = tt.cfgFile
 			// Backup original stderr
 			origStderr := os.Stderr
@@ -50,6 +63,7 @@ func Test_initConfig(t *testing.T) {
 			if _, err := io.Copy(&buf, r); err != nil {
 				t.Fatal(err)
 			}
+
 			capturedStderr := buf.String()
 
 			// Now you can assert capturedStderr
