@@ -1782,54 +1782,42 @@ func TestDocument_specifiedAlign(t *testing.T) {
 		})
 	}
 }
+func TestDocument_closeFile(t *testing.T) {
+	t.Parallel()
 
-func TestRoot_closeFile(t *testing.T) {
-	tcellNewScreen = fakeScreen
-	defer func() {
-		tcellNewScreen = tcell.NewScreen
-	}()
-	type fields struct {
-		seekable bool
-	}
 	tests := []struct {
-		name    string
-		fields  fields
-		wantErr bool
-		message string
+		name     string
+		seekable bool
+		wantErr  bool
 	}{
 		{
-			name: "testCloseFileSuccess",
-			fields: fields{
-				seekable: false,
-			},
-			wantErr: false,
-			message: "close file",
+			name:     "testCloseNonSeekableFile",
+			seekable: false,
+			wantErr:  false,
 		},
 		{
-			name: "testCloseFileError",
-			fields: fields{
-				seekable: true,
-			},
-			wantErr: true,
-			message: "cannot close",
+			name:     "testCloseSeekableFile",
+			seekable: true,
+			wantErr:  true, // seekable file close returns error in test
 		},
 	}
+
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
-			root := rootHelper(t)
-			ctx := context.Background()
-			root.Doc.seekable = tt.fields.seekable
-			err := root.Doc.closeFile()
+			t.Parallel()
+
+			doc, err := OpenDocument(filepath.Join(testdata, "test.txt"))
+			if err != nil {
+				t.Fatal(err)
+			}
+			defer doc.close()
+
+			doc.seekable = tt.seekable
+
+			err = doc.closeFile()
 			if (err != nil) != tt.wantErr {
 				t.Errorf("closeFile() error = %v, wantErr %v", err, tt.wantErr)
 			}
-			root.closeFile(ctx)
-			if tt.wantErr && root.message == "" {
-				t.Errorf("expected error message log, got empty")
-			}
-			if !strings.Contains(root.message, tt.message) {
-				t.Errorf("expected message %v, got %v", tt.message, root.message)
-			}
 		})
 	}
 }
@@ -2271,6 +2259,7 @@ func TestDocument_toggleRightAlign(t *testing.T) {
 		})
 	}
 }
+
 func TestRoot_setViewMode(t *testing.T) {
 	root := rootHelper(t)
 	ctx := context.Background()
@@ -2371,3 +2360,27 @@ func TestRoot_setViewMode(t *testing.T) {
 		})
 	}
 }
+
+func TestRoot_toggleStatusLine(t *testing.T) {
+	root := rootHelper(t)
+	ctx := context.Background()
+
+	// Initially StatusLine should be false
+	root.Doc.StatusLine = false
+	root.toggleStatusLine(ctx)
+	if !root.Doc.StatusLine {
+		t.Errorf("toggleStatusLine() StatusLine = %v, want %v", root.Doc.StatusLine, true)
+	}
+	if root.message != "Status Line visible" {
+		t.Errorf("toggleStatusLine() message = %v, want %v", root.message, "Status Line visible")
+	}
+
+	// Toggle again, should be false
+	root.toggleStatusLine(ctx)
+	if root.Doc.StatusLine {
+		t.Errorf("toggleStatusLine() StatusLine = %v, want %v", root.Doc.StatusLine, false)
+	}
+	if root.message != "Status Line hidden" {
+		t.Errorf("toggleStatusLine() message = %v, want %v", root.message, "Status Line hidden")
+	}
+}
