@@ -3,7 +3,7 @@ package oviewer
 import (
 	"regexp"
 
-	"github.com/mattn/go-runewidth"
+	"github.com/rivo/uniseg"
 )
 
 // align is a converter that aligns columns.
@@ -80,7 +80,7 @@ func (a *align) convert(st *parseState) bool {
 		return false
 	}
 	a.count += 1
-	if runewidth.RuneWidth(st.mainc) > 1 {
+	if uniseg.StringWidth(string(append([]rune{st.mainc}, st.combc...))) > 1 {
 		a.count += 1
 	}
 
@@ -110,7 +110,7 @@ func (a *align) convertDelm(src contents) contents {
 	}
 	dst := make(contents, 0, len(src))
 	start := pos.x(0)
-	for columnNum := 0; columnNum < len(indexes); columnNum++ {
+	for columnNum := range indexes {
 		end := pos.x(indexes[columnNum][0]) // Start of the delimiter.
 		delmEnd := pos.x(indexes[columnNum][1])
 		if a.isShrink(columnNum) {
@@ -140,7 +140,7 @@ func (a *align) convertWidth(src contents) contents {
 	dst := make(contents, 0, len(src))
 
 	start := 0
-	for columnNum := 0; columnNum < len(a.orgWidths); columnNum++ {
+	for columnNum := range a.orgWidths {
 		end := findColumnEnd(src, a.orgWidths, columnNum, start) + 1
 		end = min(end, len(src))
 
@@ -148,7 +148,7 @@ func (a *align) convertWidth(src contents) contents {
 		if a.isShrink(columnNum) {
 			dst = appendShrink(dst)
 			dst = append(dst, SpaceContent)
-			a.maxWidths[columnNum] = runewidth.RuneWidth(Shrink)
+			a.maxWidths[columnNum] = uniseg.StringWidth(string(Shrink))
 			start = end
 			continue
 		}
@@ -207,7 +207,7 @@ func appendShrink(lc contents) contents {
 
 // appendPaddings adds spaces to the lc.
 func appendPaddings(lc contents, num int) contents {
-	for i := 0; i < num; i++ {
+	for range num {
 		lc = append(lc, SpaceContent)
 	}
 	return lc
