@@ -5,54 +5,52 @@ import (
 	"fmt"
 	"log"
 	"slices"
+	"strings"
 	"time"
 
 	"github.com/gdamore/tcell/v2"
+	"github.com/mattn/go-runewidth"
 )
 
 // defaultStyle is used when the style is not specified.
 var defaultStyle = tcell.StyleDefault
 
-// draw is the main routine that draws the screen.
-func (root *Root) draw(ctx context.Context) {
-	m := root.Doc
-
-	// Prepare the screen for drawing.
-	root.prepareDraw(ctx)
-
-	root.drawRuler()
-	// Body.
-	lX := m.topLX
-	lN := m.topLN + root.scr.headerEnd
-	// If WrapMode is off, lX is always 0.
-	if !m.WrapMode {
-		lX = 0
-	}
-	lX, lN = root.drawBody(lX, lN)
-	m.bottomLX = lX
-	m.bottomLN = lN
-
-	// Header.
-	if m.headerHeight > 0 {
-		root.drawHeader()
-	}
-	// Section header.
-	if m.sectionHeaderHeight > 0 {
-		root.drawSectionHeader()
-	}
-
-	if root.scr.mouseSelect != SelectNone {
-		root.drawSelect(root.scr.x1, root.scr.y1, root.scr.x2, root.scr.y2, root.scr.mouseSelect)
-	}
-
-	root.drawStatus()
-	root.drawTitle()
-	root.Screen.Show()
+var anchorPointStyle = OVStyle{
+	Reverse: true,
 }
 
-// drawBody draws body.
-func (root *Root) drawBody(lX int, lN int) (int, int) {
+// draw is the main routine that draws the screen.
+func (root *Root) draw(ctx context.Context) {
+	shouldSync := root.scr.forceDisplaySync
+	root.scr.forceDisplaySync = false
+	defer func() {
+		if shouldSync {
+			root.Screen.Sync()
+		} else {
+			root.Screen.Show()
+		}
+	}()
+
+	root.prepareDraw(ctx)
+	root.drawBody()
+
+	root.drawRuler()
+	root.drawHeader()
+	root.drawSectionHeader()
+	root.drawSelect()
+	root.drawStatus()
+	root.drawTitle()
+}
+
+// drawBody draws the body.
+// drawBody sets bottomLN and bottomLX of the document.
+func (root *Root) drawBody() {
 	m := root.Doc
+	lX := m.topLX
+	lN := m.topLN + root.scr.headerEnd
+	if !m.WrapMode { // If WrapMode is off, topLX is always 0.
+		lX = 0
+	}
 
 	markStyleWidth := min(root.scr.vWidth, root.Doc.MarkStyleWidth)
 
@@ -88,13 +86,16 @@ func (root *Root) drawBody(lX int, lN int) (int, int) {
 		lX = nextLX
 		lN = nextLN
 	}
-	return lX, lN
+	m.bottomLN = lN
+	m.bottomLX = lX
 }
 
 // drawHeader draws header.
 func (root *Root) drawHeader() {
 	m := root.Doc
-
+	if m.headerHeight == 0 {
+		return
+	}
 	// wrapNum is the number of wrapped lines.
 	wrapNum := 0
 	lX := 0
@@ -126,7 +127,9 @@ func (root *Root) drawHeader() {
 // The section header overrides the body.
 func (root *Root) drawSectionHeader() {
 	m := root.Doc
-
+	if m.sectionHeaderHeight == 0 {
+		return
+	}
 	wrapNum := 0
 	lX := 0
 	lN := root.scr.sectionHeaderLN
@@ -143,7 +146,7 @@ func (root *Root) drawSectionHeader() {
 		root.applyStyleToLine(y, m.Style.SectionLine)
 
 		root.drawVerticalHeader(y, wrapNum, lineC)
-		// markstyle is displayed above the section header.
+		// markStyle is displayed above the section header.
 		markStyleWidth := min(root.scr.vWidth, m.MarkStyleWidth)
 		root.applyMarkStyle(lN, y, markStyleWidth)
 		// Underline search lines when they overlap in section headers.
@@ -162,6 +165,12 @@ func (root *Root) drawSectionHeader() {
 	root.applyStyleToLine(m.headerHeight+m.sectionHeaderHeight-1, m.Style.SectionHeaderBorder)
 }
 
+const (
+	minusRuler = "0"
+	tenRuler   = "         1         2         3         4         5         6         7         8         9         0"
+	oneRuler   = "1234567890"
+)
+
 // drawRuler draws the ruler.
 func (root *Root) drawRuler() {
 	rulerType := root.Doc.RulerType
@@ -169,26 +178,23 @@ func (root *Root) drawRuler() {
 		return
 	}
 
+	style := applyStyle(defaultStyle, root.Doc.Style.Ruler)
+
 	startX := 0
+	offset := 0
 	if !root.Doc.WrapMode && rulerType == RulerRelative {
 		startX = root.scr.startX - root.Doc.x
-	}
-
-	style := applyStyle(defaultStyle, root.Doc.Style.Ruler)
-	for x := range root.scr.vWidth {
-		n := x - startX + 1
-		if n < 0 {
-			continue
+		if startX < 0 {
+			offset = -startX
+			startX = 0
 		}
-		numStr := []rune(fmt.Sprintf("%3d", n))
-		if numStr[2] == '0' {
-			root.Screen.SetContent(x-1, 0, numStr[0], nil, style)
-			root.Screen.SetContent(x, 0, numStr[1], nil, style)
-		} else {
-			root.Screen.SetContent(x, 0, ' ', nil, style)
-		}
-		root.Screen.SetContent(x, 1, numStr[2], nil, style)
 	}
+	root.Screen.PutStrStyled(0, 0, strings.Repeat(" ", startX), style)
+	root.Screen.PutStrStyled(0, 1, strings.Repeat(minusRuler, startX), style)
+	tensLine := strings.Repeat(tenRuler, (root.scr.vWidth+offset)/100+1)
+	onesLine := strings.Repeat(oneRuler, (root.scr.vWidth+offset)/10+1)
+	root.Screen.PutStrStyled(startX, 0, tensLine[offset:], style)
+	root.Screen.PutStrStyled(startX, 1, onesLine[offset:], style)
 }
 
 // drawWrapLine wraps and draws the contents and returns the next drawing position.
@@ -206,7 +212,6 @@ func (root *Root) drawWrapLine(y int, lX int, lN int, lineC LineC) (int, int) {
 		log.Printf("Illegal lX: %d\n", lX)
 		return 0, 0
 	}
-	screen := root.Screen
 	for n := 0; ; n++ {
 		x := root.scr.startX + n
 		if lX+n >= len(lineC.lc) {
@@ -223,7 +228,7 @@ func (root *Root) drawWrapLine(y int, lX int, lN int, lineC LineC) (int, int) {
 			lX += n
 			break
 		}
-		screen.SetContent(x, y, c.mainc, c.combc, c.style)
+		root.setContent(x, y, c.mainc, c.combc, c.style)
 		if c.width == 2 {
 			n++
 		}
@@ -234,7 +239,6 @@ func (root *Root) drawWrapLine(y int, lX int, lN int, lineC LineC) (int, int) {
 // drawNoWrapLine draws contents without wrapping and returns the next drawing position.
 func (root *Root) drawNoWrapLine(y int, lX int, lN int, lineC LineC) (int, int) {
 	lX = max(lX, root.minStartX)
-	screen := root.Screen
 	for n := 0; root.scr.startX+n < root.scr.vWidth; n++ {
 		x := root.scr.startX + n
 		if lX+n >= len(lineC.lc) {
@@ -243,12 +247,11 @@ func (root *Root) drawNoWrapLine(y int, lX int, lN int, lineC LineC) (int, int)
 			break
 		}
 		if lX+n < 0 {
-			c := DefaultContent
-			screen.SetContent(x, y, c.mainc, c.combc, c.style)
+			root.Screen.Put(x, y, " ", defaultStyle)
 			continue
 		}
 		c := lineC.lc[lX+n]
-		screen.SetContent(x, y, c.mainc, c.combc, c.style)
+		root.setContent(x, y, c.mainc, c.combc, c.style)
 		if c.width == 2 {
 			n++
 		}
@@ -273,24 +276,19 @@ func (root *Root) drawVerticalHeader(y int, wrapNum int, lineC LineC) {
 		widthVH--
 	}
 
-	screen := root.Screen
 	x := root.scr.startX
 	for n := 0; n < widthVH; n++ {
 		c := DefaultContent
 		if n < len(lineC.lc) {
 			c = lineC.lc[n]
 		}
-		if n == widthVH-2 && c.width == 2 {
-			style := applyStyle(defaultStyle, root.Doc.Style.VerticalHeaderBorder)
-			screen.SetContent(x, y, c.mainc, c.combc, style)
-			return
-		} else if n == widthVH-1 {
-			style := applyStyle(defaultStyle, root.Doc.Style.VerticalHeaderBorder)
-			screen.SetContent(x, y, c.mainc, c.combc, style)
-			return
+		var style tcell.Style
+		if n == widthVH-1 || (n == widthVH-2 && c.width == 2) {
+			style = applyStyle(defaultStyle, root.Doc.Style.VerticalHeaderBorder)
+		} else {
+			style = applyStyle(c.style, root.Doc.Style.VerticalHeader)
 		}
-		style := applyStyle(c.style, root.Doc.Style.VerticalHeader)
-		screen.SetContent(x, y, c.mainc, c.combc, style)
+		root.setContent(x, y, c.mainc, c.combc, style)
 		x++
 		if c.width == 2 {
 			x++
@@ -328,7 +326,7 @@ func (root *Root) blankLineNumber(y int) {
 		return
 	}
 	for x := range root.scr.startX {
-		root.Screen.SetContent(x, y, ' ', nil, defaultStyle)
+		root.Screen.PutStr(x, y, " ")
 	}
 }
 
@@ -357,11 +355,8 @@ func (root *Root) drawLineNumber(lN int, y int, valid bool) {
 	number = number - m.firstLine() + 1
 
 	style := applyStyle(defaultStyle, m.Style.LineNumber)
-	numC := []rune(fmt.Sprintf("%*d", root.scr.startX-1, number))
-	for i := range numC {
-		root.Screen.SetContent(i, y, numC[i], nil, style)
-	}
-	root.Screen.SetContent(len(numC), y, ' ', nil, defaultStyle)
+	numC := fmt.Sprintf("%*d ", root.scr.startX-1, number)
+	root.Screen.PutStrStyled(0, y, numC, style)
 }
 
 // drawTitle sets the terminal title if TerminalTitle is enabled.
@@ -379,25 +374,12 @@ func (root *Root) displayTitle() string {
 	return root.Doc.FileName
 }
 
-// setContentString is a helper function that draws a string with setContent.
-func (root *Root) setContentString(vx int, vy int, lc contents) {
-	screen := root.Screen
-	x := vx
-	for _, content := range lc {
-		screen.SetContent(x, vy, content.mainc, content.combc, content.style)
-		x++
-		if content.width == 2 {
-			x++
-		}
-	}
-	screen.SetContent(x, vy, 0, nil, defaultStyle.Normal())
-}
-
 // clearEOL clears from the specified position to the right end.
 func (root *Root) clearEOL(x int, y int, style tcell.Style) {
-	for ; x < root.scr.vWidth; x++ {
-		root.Screen.SetContent(x, y, ' ', nil, style)
+	if x >= root.scr.vWidth {
+		return
 	}
+	root.Screen.PutStrStyled(x, y, strings.Repeat(" ", root.scr.vWidth-x), style)
 }
 
 // clearY clear the specified line.
@@ -444,10 +426,10 @@ func (root *Root) applyMarkStyle(lN int, y int, width int) {
 // applyStyleToRange applies the style from the start to the end of the physical line.
 func (root *Root) applyStyleToRange(y int, s OVStyle, start int, end int) {
 	for x := start; x < end; x++ {
-		mainc, combc, style, width := root.Screen.GetContent(x, y)
+		str, style, width := root.Screen.Get(x, y)
 		newStyle := applyStyle(style, s)
 		if style != newStyle {
-			root.Screen.SetContent(x, y, mainc, combc, newStyle)
+			root.Screen.Put(x, y, str, newStyle)
 		}
 		if width == 2 {
 			x++
@@ -477,10 +459,28 @@ func (root *Root) hideOtherSection(y int, line LineC) {
 	root.clearY(y)
 }
 
-// drawSelect highlights the selection.
+// drawSelect highlights the mouse selection.
 // Multi-line selection is included until the end of the line,
 // but if the rectangle flag is true, the rectangle will be the range.
-func (root *Root) drawSelect(x1, y1, x2, y2 int, sel MouseSelectState) {
+func (root *Root) drawSelect() {
+	sel := root.scr.mouseSelect
+	x1 := root.scr.x1
+	y1 := root.scr.y1
+	x2 := root.scr.x2
+	y2 := root.scr.y2
+
+	if root.scr.hasAnchorPoint {
+		// Highlight the anchor point if the mouse was clicked but no selection is active.
+		if x1 == x2 && y1 == y2 {
+			root.applyStyleToRange(y1, anchorPointStyle, x1, x2+1)
+			return
+		}
+	}
+
+	if sel == SelectNone {
+		return
+	}
+
 	if y2 < y1 {
 		y1, y2 = y2, y1
 		x1, x2 = x2, x1
@@ -523,41 +523,17 @@ func (root *Root) applySelectionRange(y int, start int, end int, selectState Mou
 	}
 
 	for x := start; x < end; x++ {
-		mainc, combc, style, width := root.Screen.GetContent(x, y)
+		str, style, width := root.Screen.Get(x, y)
 		newStyle := applyStyle(style, selectStyle)
-		root.Screen.SetContent(x, y, mainc, combc, newStyle)
+		root.Screen.Put(x, y, str, newStyle)
 		if width == 2 {
 			x++
 		}
 	}
 }
 
-// quitCheck checks if it should quit when the document is small.
-func (root *Root) quitCheck() bool {
-	if !root.Config.QuitSmall {
-		return false
-	}
-	if !root.docSmall() {
-		return false
-	}
-	if root.DocumentLen() == 1 && !root.Config.QuitSmallFilter {
-		root.Config.IsWriteOnExit = true
-		return true
-	}
-	return false
-}
-
-// notifyEOFReached notifies that EOF has been reached.
-func (root *Root) notifyEOFReached(m *Document) {
-	if root.Config.NotifyEOF == 0 {
-		return
-	}
-	root.setMessagef("EOF reached %s", m.FileName)
-	root.notify(root.Config.NotifyEOF)
-}
-
-// notify notifies by beeping and flashing the screen the specified number of times.
-func (root *Root) notify(count int) {
+// execNotify notifies by beeping and flashing the screen the specified number of times.
+func (root *Root) execNotify(count int) {
 	for range count {
 		_ = root.Screen.Beep()
 		root.flash()
@@ -575,3 +551,41 @@ func (root *Root) flash() {
 	root.draw(context.Background())
 	time.Sleep(100 * time.Millisecond)
 }
+
+// setContent wraps Screen.SetContent and checks for display sync needs.
+func (root *Root) setContent(x, y int, mainc rune, combc []rune, style tcell.Style) {
+	if !root.scr.forceDisplaySync && needsDisplaySync(mainc, combc) {
+		root.scr.forceDisplaySync = true
+	}
+	root.Screen.SetContent(x, y, mainc, combc, style)
+}
+
+// needsDisplaySync checks if display sync handling is needed.
+// This is required when there's a mismatch between the character width
+// recognized by tcell and the actual display width in the terminal.
+// When this mismatch occurs, the display can become corrupted, so we
+// need to use Screen.Sync() instead of Screen.Show() to ensure proper rendering.
+//
+// Common cases that require sync:
+// - Combining characters (ZWJ sequences, variation selectors, skin tone modifiers)
+// - Ambiguous width characters (box drawing, mathematical symbols, etc.)
+// - Regional indicator symbols used in flag emojis (🇺🇸, 🇯🇵, etc.)
+// - Complex character sequences that may be rendered differently across terminals
+func needsDisplaySync(mainc rune, combc []rune) bool {
+	// Any combining characters can cause display issues
+	if len(combc) > 0 {
+		return true
+	}
+
+	// Check for ambiguous width characters using runewidth library
+	if runewidth.IsAmbiguousWidth(mainc) {
+		return true
+	}
+	// Regional indicator symbols and emoji range that commonly cause issues
+	// Check for regional indicator symbols (used in flag emojis).
+	if mainc >= 0x1F1E0 && mainc <= 0x1F9FF {
+		return true
+	}
+
+	return false
+}
