@@ -6,14 +6,14 @@ import (
 	"github.com/gdamore/tcell/v2"
 )
 
-// setHeaderMode sets the inputMode to Header.
-func (root *Root) setHeaderMode(context.Context) {
+// inputHeader sets the inputMode to Header.
+func (root *Root) inputHeader(context.Context) {
 	input := root.input
 	input.reset()
 	input.Event = newHeaderEvent()
 }
 
-// eventHeader represents the goto input mode.
+// eventHeader represents the header input mode.
 type eventHeader struct {
 	tcell.EventTime
 	value string
