@@ -250,10 +250,10 @@ func defaultKeyBinds() KeyBind {
 		actionPreviousDoc:    {"["},
 		actionCloseDoc:       {"ctrl+k"},
 		actionCloseAllFilter: {"K"},
-		actionToggleMouse:    {"ctrl+alt+r"},
+		actionToggleMouse:    {"ctrl+F8", "ctrl+alt+r"},
 		actionHideOther:      {"alt+-"},
-		actionAlignFormat:    {"alt+F"},
-		actionRawFormat:      {"alt+R"},
+		actionAlignFormat:    {"alt+f"},
+		actionRawFormat:      {"alt+r"},
 		actionFixedColumn:    {"F"},
 		actionShrinkColumn:   {"s"},
 		actionRightAlign:     {"alt+a"},
@@ -262,11 +262,11 @@ func defaultKeyBinds() KeyBind {
 		actionStatusLine:     {"ctrl+F10"},
 
 		// Move actions.
-		actionMoveDown:       {"Enter", "Down", "ctrl+N"},
+		actionMoveDown:       {"Enter", "Down", "ctrl+n"},
 		actionMoveUp:         {"Up", "ctrl+p"},
 		actionMoveTop:        {"Home"},
-		actionMoveWidthLeft:  {"ctrl+shift+left"},
-		actionMoveWidthRight: {"ctrl+shift+right"},
+		actionMoveWidthLeft:  {"alt+left"},
+		actionMoveWidthRight: {"alt+right"},
 		actionMoveLeft:       {"left"},
 		actionMoveRight:      {"right"},
 		actionMoveHfLeft:     {"ctrl+left"},
@@ -516,20 +516,6 @@ func setHandler(ctx context.Context, c *cbind.Configuration, name string, keys [
 		}
 
 		c.SetKey(mod, key, wrapEventHandler(ctx, handler))
-
-		// Special case: if ctrl+j is bound, also bind Enter to the same action.
-		if key == tcell.KeyEnter && strings.ToLower(k) == "ctrl+j" {
-			// Also bind to actual Ctrl+J event
-			c.SetKey(tcell.ModCtrl, tcell.KeyCtrlJ, wrapEventHandler(ctx, handler))
-		}
-
-		// ctrl+h, Backspace and ctrl+Backspace can only be assigned one handler.
-		if key == tcell.KeyBackspace || key == tcell.KeyBackspace2 {
-			c.SetKey(tcell.ModNone, tcell.KeyBackspace, wrapEventHandler(ctx, handler))
-			c.SetKey(tcell.ModNone, tcell.KeyBackspace2, wrapEventHandler(ctx, handler))
-			c.SetKey(tcell.ModCtrl, tcell.KeyBackspace, wrapEventHandler(ctx, handler))
-			c.SetKey(tcell.ModCtrl, tcell.KeyBackspace2, wrapEventHandler(ctx, handler))
-		}
 	}
 	return nil
 }
@@ -544,8 +530,28 @@ func wrapEventHandler(ctx context.Context, f func(context.Context)) func(_ *tcel
 
 // keyCapture does the actual key action.
 func (root *Root) keyCapture(ev *tcell.EventKey) bool {
-	root.keyConfig.Capture(ev)
-	return true
+	if root.keyConfig.Capture(ev) == nil {
+		return true
+	}
+	if root.Config.Debug {
+		root.setMessageLogf("key \"%s\" not assigned", root.formatKeyName(ev))
+	}
+	return false
+}
+
+// formatKeyName formats the key name for better readability
+func (root *Root) formatKeyName(ev *tcell.EventKey) string {
+	// For regular printable characters, just show the character
+	if ev.Rune() != 0 && ev.Rune() > 32 && ev.Rune() < 127 {
+		// Check for modifiers
+		if ev.Modifiers() != tcell.ModNone {
+			return ev.Name() // Use the full name for modified keys
+		}
+		return string(ev.Rune()) // Just show the character for simple keys
+	}
+
+	// For special keys, use the standard name
+	return ev.Name()
 }
 
 // keyActionMapping represents a key that is assigned to multiple actions.
@@ -601,7 +607,7 @@ func findDuplicateKeyBind(keyBind KeyBind) ([]keyActionMapping, []error) {
 		for _, key := range keys {
 			normalizedKey, err := normalizeKeyWithPrefix(key, action)
 			if err != nil {
-				errors = append(errors, fmt.Errorf("key %s for action %s", key, action))
+				errors = append(errors, fmt.Errorf("%w: key %s for action %s", ErrInvalidKey, key, action))
 				continue // Skip this key and continue with the next one.
 			}
 
