@@ -9,9 +9,11 @@ import (
 
 	"github.com/atotto/clipboard"
 	"github.com/gdamore/tcell/v2"
-	"github.com/mattn/go-runewidth"
 )
 
+// clipboardTimeout is the timeout duration for clipboard operations.
+const clipboardTimeout = 2 * time.Second
+
 // ClickType represents the type of mouse click.
 type ClickType int
 
@@ -94,7 +96,6 @@ func (root *Root) wheelUp(context.Context) {
 // wheelDown moves the mouse wheel down.
 func (root *Root) wheelDown(context.Context) {
 	root.setMessage("")
-	log.Println("vScrollLines:", root.Doc.VScrollLines)
 	root.moveDown(root.Doc.VScrollLines)
 }
 
@@ -143,11 +144,15 @@ func (root *Root) mouseSelect(ctx context.Context, ev *tcell.EventMouse) bool {
 
 // handleSelectNone handles mouse events when no selection is active.
 func (root *Root) handleSelectNone(ctx context.Context, ev *tcell.EventMouse, button tcell.ButtonMask) bool {
-	if button != tcell.ButtonPrimary {
-		return false
+	if button == tcell.ButtonPrimary {
+		return root.handlePrimaryButtonClick(ctx, ev)
 	}
 
-	return root.handlePrimaryButtonClick(ctx, ev)
+	if button == tcell.ButtonSecondary {
+		return root.handleSecondaryButtonClick(ctx, ev)
+	}
+
+	return false
 }
 
 // handleSelectCopied handles mouse events when selection is copied.
@@ -164,8 +169,16 @@ func (root *Root) handleSelectCopied(ctx context.Context, ev *tcell.EventMouse,
 // handlePrimaryButtonClick handles single/double/triple click events.
 func (root *Root) handlePrimaryButtonClick(ctx context.Context, ev *tcell.EventMouse) bool {
 	x, y := ev.Position()
+	mod := ev.Modifiers()
 	now := time.Now()
 
+	// Alt+click extension has priority over click type detection
+	if mod&tcell.ModAlt != 0 && root.scr.hasAnchorPoint {
+		// Update click state for Alt+click (to maintain consistency)
+		root.updateClickState(x, y, now)
+		return root.extendSelectionWithAltClick(ev)
+	}
+
 	clickType := root.checkClickType(x, y, now)
 
 	switch clickType {
@@ -189,12 +202,35 @@ func (root *Root) handlePrimaryButtonClick(ctx context.Context, ev *tcell.EventM
 	return false
 }
 
+func (root *Root) handleSecondaryButtonClick(ctx context.Context, ev *tcell.EventMouse) bool {
+	if !root.scr.hasAnchorPoint {
+		return false
+	}
+	x, y := ev.Position()
+	root.scr.x2, root.scr.y2 = root.adjustPositionForWideChar(x, y)
+	root.CopySelect(ctx)
+	root.scr.mousePressed = false
+	return true
+}
+
+// extendSelectionWithAltClick extends the current selection to the Alt+click position.
+// This implements the anchor-and-extend selection workflow where the first click
+// sets an anchor point and Alt+click extends the selection to the new position.
+func (root *Root) extendSelectionWithAltClick(ev *tcell.EventMouse) bool {
+	x, y := ev.Position()
+	root.scr.x2, root.scr.y2 = root.adjustPositionForWideChar(x, y)
+	root.scr.mouseSelect = SelectActive
+	return true
+}
+
 // startSelection initializes a new selection.
 func (root *Root) startSelection(x, y int, modifiers tcell.ModMask) {
+	x, y = root.adjustPositionForWideChar(x, y)
 	root.scr.x1, root.scr.y1 = x, y
 	root.scr.x2, root.scr.y2 = x, y
 
 	root.scr.mouseSelect = SelectActive
+	root.scr.hasAnchorPoint = true
 	root.scr.mousePressed = true
 	root.scr.mouseRectangle = false
 	// If Ctrl is pressed, enable rectangle selection
@@ -207,17 +243,17 @@ func (root *Root) startSelection(x, y int, modifiers tcell.ModMask) {
 func (root *Root) handleSelectActive(ctx context.Context, ev *tcell.EventMouse, button tcell.ButtonMask) bool {
 	switch button {
 	case tcell.ButtonPrimary:
-		root.selectRange(ctx, ev)
+		x, y := ev.Position()
+		root.scr.x2, root.scr.y2 = root.adjustPositionForWideChar(x, y)
 		return true
 	case tcell.ButtonNone:
 		// Check if mouse has moved enough to be considered a selection
-		if root.hasMinimumSelection() {
-			root.scr.mouseSelect = SelectCopied
-			root.scr.mousePressed = false
+		if root.scr.x1 != root.scr.x2 || root.scr.y1 != root.scr.y2 {
 			root.CopySelect(ctx)
 		} else {
-			// Reset selection if no meaningful movement occurred
-			root.resetSelect()
+			// No selection made, reset
+			root.scr.mouseSelect = SelectNone
+			root.scr.mousePressed = false
 		}
 		return true
 	default:
@@ -225,6 +261,18 @@ func (root *Root) handleSelectActive(ctx context.Context, ev *tcell.EventMouse,
 	}
 }
 
+// adjustPositionForWideChar adjusts the position if a wide character is clicked.
+func (root *Root) adjustPositionForWideChar(x, y int) (int, int) {
+	p, _, _, _ := root.Screen.GetContent(x, y)
+	if p == ' ' && x > 0 {
+		_, _, _, w := root.Screen.GetContent(x-1, y) // Adjust for clicking the second half of a wide character
+		if w == 2 {
+			x--
+		}
+	}
+	return x, y
+}
+
 // resetClickState resets the click state.
 func (root *Root) resetClickState() {
 	root.clickState.clickCount = 0
@@ -261,6 +309,13 @@ func (root *Root) checkClickType(x, y int, now time.Time) ClickType {
 // handleDoubleClick handles double click events for word/column selection (triple click uses handleTripleClick).
 func (root *Root) handleDoubleClick(ctx context.Context, ev *tcell.EventMouse) {
 	root.doubleClickSetRange(ev)
+
+	// Check if a valid selection range was created
+	if root.scr.x1 == root.scr.x2 && root.scr.y1 == root.scr.y2 {
+		// No selection range found, clear anchor and reset
+		root.resetSelect()
+		return
+	}
 	root.scr.mouseSelect = SelectCopied
 	root.scr.mousePressed = false
 	root.CopySelect(ctx)
@@ -303,7 +358,7 @@ func (root *Root) updateClickState(x, y int, now time.Time) {
 // findColumnBoundaries selects the entire column at the given position.
 func (root *Root) findColumnBoundaries(x, y int, lineC LineC, ln LineNumber) (int, int, int, int) {
 	contentX := root.Doc.x + (x - root.scr.startX) + root.scr.branchWidth(lineC.lc, ln.wrap)
-	startCol, endCol, ok := root.findColumnRange(contentX, lineC.columnRanges)
+	startCol, endCol, ok := findColumnRange(contentX, lineC.columnRanges)
 	if !ok {
 		return x, y, x, y
 	}
@@ -332,16 +387,6 @@ func (root *Root) findColumnBoundaries(x, y int, lineC LineC, ln LineNumber) (in
 	return startX, startY, endX, endY
 }
 
-// findColumnRange returns the start and end of the column range containing contentX, or ok=false if not found.
-func (root *Root) findColumnRange(contentX int, columnRanges []columnRange) (start, end int, ok bool) {
-	for _, colRange := range columnRanges {
-		if contentX >= colRange.start && contentX <= colRange.end {
-			return colRange.start, colRange.end, true
-		}
-	}
-	return 0, 0, false
-}
-
 // findWordBoundariesInLine selects the word at the given position.
 func (root *Root) findWordBoundariesInLine(x, y int, lineC LineC, ln LineNumber) (int, int, int, int) {
 	x = x - root.scr.startX
@@ -350,12 +395,12 @@ func (root *Root) findWordBoundariesInLine(x, y int, lineC LineC, ln LineNumber)
 		// Out of bounds: return clicked position only
 		return x + root.scr.startX, y, x + root.scr.startX, y
 	}
-	charType := root.getCharTypeAt(lineC, contentX)
+	charType := getCharTypeAt(lineC, contentX)
 	startX := x
 	startY := y
 	for ln.wrap >= 0 {
 		testContentX := (startX - 1) + root.scr.branchWidth(lineC.lc, ln.wrap)
-		if root.getCharTypeAt(lineC, testContentX) != charType {
+		if getCharTypeAt(lineC, testContentX) != charType {
 			break
 		}
 		startX--
@@ -369,7 +414,7 @@ func (root *Root) findWordBoundariesInLine(x, y int, lineC LineC, ln LineNumber)
 	endY := startY
 	for endX < len(lineC.lc)-1 {
 		testContentX := (endX + 1) + root.scr.branchWidth(lineC.lc, ln.wrap)
-		if root.getCharTypeAt(lineC, testContentX) != charType {
+		if getCharTypeAt(lineC, testContentX) != charType {
 			break
 		}
 		endX++
@@ -382,13 +427,31 @@ func (root *Root) findWordBoundariesInLine(x, y int, lineC LineC, ln LineNumber)
 	return startX + root.scr.startX, startY, endX + root.scr.startX, endY
 }
 
+// findColumnRange returns the start and end of the column range containing contentX, or ok=false if not found.
+func findColumnRange(contentX int, columnRanges []columnRange) (start, end int, ok bool) {
+	for _, colRange := range columnRanges {
+		if contentX >= colRange.start && contentX <= colRange.end {
+			return colRange.start, colRange.end, true
+		}
+	}
+	return 0, 0, false
+}
+
 // getCharTypeAt returns character type at the given content position.
-func (root *Root) getCharTypeAt(line LineC, contentX int) int {
+func getCharTypeAt(line LineC, contentX int) int {
 	if contentX < 0 || contentX >= len(line.lc) {
 		return charTypeWhitespace // Out of bounds treated as whitespace
 	}
-
-	char := line.lc[contentX].mainc
+	content := line.lc[contentX]
+	char := content.mainc
+	width := content.width
+	if width == 0 && char == 0 {
+		if contentX == 0 {
+			return charTypeOther
+		}
+		// If width is 0, check the previous character.
+		char = line.lc[contentX-1].mainc
+	}
 	if char == ' ' || char == '\t' {
 		return charTypeWhitespace
 	}
@@ -398,18 +461,6 @@ func (root *Root) getCharTypeAt(line LineC, contentX int) int {
 	return charTypeOther
 }
 
-// hasMinimumSelection checks if the selection is large enough to be meaningful.
-func (root *Root) hasMinimumSelection() bool {
-	// Require movement of at least 1 position in either direction
-	return root.scr.x1 != root.scr.x2 || root.scr.y1 != root.scr.y2
-}
-
-// selectRange saves the position by selecting the range with the mouse.
-// The selection range is represented by (x1, y1), (x2, y2).
-func (root *Root) selectRange(_ context.Context, ev *tcell.EventMouse) {
-	root.scr.x2, root.scr.y2 = ev.Position()
-}
-
 // handleTripleClick handles triple click events (selects the whole line).
 func (root *Root) handleTripleClick(ctx context.Context, ev *tcell.EventMouse) {
 	x, y := ev.Position()
@@ -440,6 +491,7 @@ func (root *Root) resetSelect() {
 	root.scr.x1, root.scr.y1 = 0, 0
 	root.scr.x2, root.scr.y2 = 0, 0
 	root.scr.mousePressed = false
+	root.scr.hasAnchorPoint = false
 	root.setMessage("")
 }
 
@@ -450,6 +502,9 @@ type eventCopySelect struct {
 
 // CopySelect fires the eventCopySelect event.
 func (root *Root) CopySelect(context.Context) {
+	root.scr.mouseSelect = SelectCopied
+	root.scr.mousePressed = false
+
 	root.sendCopySelect()
 }
 
@@ -483,9 +538,24 @@ func (root *Root) copyClipboard(str string) {
 	case "OSC52":
 		root.Screen.SetClipboard([]byte(str))
 	default:
-		if err := clipboard.WriteAll(str); err != nil {
-			log.Printf("copyToClipboard: %v\n", err)
+		go root.copyToClipboardAsync(str)
+	}
+}
+
+// copyToClipboardAsync copies to clipboard with timeout to prevent blocking.
+func (root *Root) copyToClipboardAsync(str string) {
+	done := make(chan error, 1)
+	go func() {
+		done <- clipboard.WriteAll(str)
+	}()
+
+	select {
+	case err := <-done:
+		if err != nil {
+			log.Printf("copyToClipboard failed: %v\n", err)
 		}
+	case <-time.After(clipboardTimeout):
+		log.Printf("copyToClipboard: timeout (clipboard command may be hanging)\n")
 	}
 }
 
@@ -517,12 +587,9 @@ func (root *Root) pasteFromClipboard(context.Context) {
 		return
 	}
 
-	pos := countToCursor(input.value, input.cursorX+1)
-	runes := []rune(input.value)
-	input.value = string(runes[:pos])
-	input.value += str
-	input.value += string(runes[pos:])
-	input.cursorX += runewidth.StringWidth(str)
+	left, right := splitAtWidth(input.value, input.cursorX)
+	input.value = left + str + right
+	input.cursorX += stringWidth(str)
 }
 
 func (root *Root) rangeToString(startX, startY, endX, endY int) (string, error) {
